@@ -7,7 +7,8 @@ The |Table| object and related proxy classes.
 from __future__ import absolute_import, print_function, unicode_literals
 
 from .blkcntnr import BlockItemContainer
-from .shared import lazyproperty, Parented, write_only_property
+from .oxml.simpletypes import ST_Merge
+from .shared import lazyproperty, Parented
 
 
 class Table(Parented):
@@ -26,7 +27,7 @@ class Table(Parented):
         gridCol = tblGrid.add_gridCol()
         for tr in self._tbl.tr_lst:
             tr.add_tc()
-        return _Column(gridCol, self._tbl, self)
+        return _Column(gridCol, self)
 
     def add_row(self):
         """
@@ -57,8 +58,16 @@ class Table(Parented):
         Return |_Cell| instance correponding to table cell at *row_idx*,
         *col_idx* intersection, where (0, 0) is the top, left-most cell.
         """
-        row = self.rows[row_idx]
-        return row.cells[col_idx]
+        cell_idx = col_idx + (row_idx * self._column_count)
+        return self._cells[cell_idx]
+
+    def column_cells(self, column_idx):
+        """
+        Sequence of cells in the column at *column_idx* in this table.
+        """
+        cells = self._cells
+        idxs = range(column_idx, len(cells), self._column_count)
+        return [cells[idx] for idx in idxs]
 
     @lazyproperty
     def columns(self):
@@ -67,6 +76,15 @@ class Table(Parented):
         """
         return _Columns(self._tbl, self)
 
+    def row_cells(self, row_idx):
+        """
+        Sequence of cells in the row at *row_idx* in this table.
+        """
+        column_count = self._column_count
+        start = row_idx * column_count
+        end = start + column_count
+        return self._cells[start:end]
+
     @lazyproperty
     def rows(self):
         """
@@ -87,6 +105,42 @@ class Table(Parented):
     def style(self, value):
         self._tblPr.style = value
 
+    @property
+    def table(self):
+        """
+        Provide child objects with reference to the |Table| object they
+        belong to, without them having to know their direct parent is
+        a |Table| object. This is the terminus of a series of `parent._table`
+        calls from an arbitrary child through its ancestors.
+        """
+        return self
+
+    @property
+    def _cells(self):
+        """
+        A sequence of |_Cell| objects, one for each cell of the layout grid.
+        If the table contains a span, one or more |_Cell| object references
+        are repeated.
+        """
+        col_count = self._column_count
+        cells = []
+        for tc in self._tbl.iter_tcs():
+            for grid_span_idx in range(tc.grid_span):
+                if tc.vMerge == ST_Merge.CONTINUE:
+                    cells.append(cells[-col_count])
+                elif grid_span_idx > 0:
+                    cells.append(cells[-1])
+                else:
+                    cells.append(_Cell(tc, self))
+        return cells
+
+    @property
+    def _column_count(self):
+        """
+        The number of grid columns in this table.
+        """
+        return self._tbl.col_count
+
     @property
     def _tblPr(self):
         return self._tbl.tblPr
@@ -125,6 +179,16 @@ class _Cell(BlockItemContainer):
         self.add_paragraph()
         return new_table
 
+    def merge(self, other_cell):
+        """
+        Return a merged cell created by spanning the rectangular region
+        having this cell and *other_cell* as diagonal corners. Raises
+        |InvalidSpanError| if the cells do not define a rectangular region.
+        """
+        tc, tc_2 = self._tc, other_cell._tc
+        merged_tc = tc.merge(tc_2)
+        return _Cell(merged_tc, self._parent)
+
     @property
     def paragraphs(self):
         """
@@ -141,7 +205,16 @@ class _Cell(BlockItemContainer):
         """
         return super(_Cell, self).tables
 
-    @write_only_property
+    @property
+    def text(self):
+        """
+        The entire contents of this cell as a string of text. Assigning
+        a string to this property replaces all existing content with a single
+        paragraph containing the assigned text in a single run.
+        """
+        return '\n'.join(p.text for p in self.paragraphs)
+
+    @text.setter
     def text(self, text):
         """
         Write-only. Set entire contents of cell to the string *text*. Any
@@ -169,18 +242,23 @@ class _Column(Parented):
     """
     Table column
     """
-    def __init__(self, gridCol, tbl, parent):
+    def __init__(self, gridCol, parent):
         super(_Column, self).__init__(parent)
         self._gridCol = gridCol
-        self._tbl = tbl
 
-    @lazyproperty
+    @property
     def cells(self):
         """
         Sequence of |_Cell| instances corresponding to cells in this column.
-        Supports ``len()``, iteration and indexed access.
         """
-        return _ColumnCells(self._tbl, self._gridCol, self)
+        return tuple(self.table.column_cells(self._index))
+
+    @property
+    def table(self):
+        """
+        Reference to the |Table| object this column belongs to.
+        """
+        return self._parent.table
 
     @property
     def width(self):
@@ -194,45 +272,12 @@ class _Column(Parented):
     def width(self, value):
         self._gridCol.w = value
 
-
-class _ColumnCells(Parented):
-    """
-    Sequence of |_Cell| instances corresponding to the cells in a table
-    column.
-    """
-    def __init__(self, tbl, gridCol, parent):
-        super(_ColumnCells, self).__init__(parent)
-        self._tbl = tbl
-        self._gridCol = gridCol
-
-    def __getitem__(self, idx):
-        """
-        Provide indexed access, (e.g. 'cells[0]')
-        """
-        try:
-            tr = self._tr_lst[idx]
-        except IndexError:
-            msg = "cell index [%d] is out of range" % idx
-            raise IndexError(msg)
-        tc = tr.tc_lst[self._col_idx]
-        return _Cell(tc, self)
-
-    def __iter__(self):
-        for tr in self._tr_lst:
-            tc = tr.tc_lst[self._col_idx]
-            yield _Cell(tc, self)
-
-    def __len__(self):
-        return len(self._tr_lst)
-
     @property
-    def _col_idx(self):
-        gridCol_lst = self._tbl.tblGrid.gridCol_lst
-        return gridCol_lst.index(self._gridCol)
-
-    @property
-    def _tr_lst(self):
-        return self._tbl.tr_lst
+    def _index(self):
+        """
+        Index of this column in its table, starting from zero.
+        """
+        return self._gridCol.gridCol_idx
 
 
 class _Columns(Parented):
@@ -253,15 +298,22 @@ class _Columns(Parented):
         except IndexError:
             msg = "column index [%d] is out of range" % idx
             raise IndexError(msg)
-        return _Column(gridCol, self._tbl, self)
+        return _Column(gridCol, self)
 
     def __iter__(self):
         for gridCol in self._gridCol_lst:
-            yield _Column(gridCol, self._tbl, self)
+            yield _Column(gridCol, self)
 
     def __len__(self):
         return len(self._gridCol_lst)
 
+    @property
+    def table(self):
+        """
+        Reference to the |Table| object this column collection belongs to.
+        """
+        return self._parent.table
+
     @property
     def _gridCol_lst(self):
         """
@@ -280,39 +332,26 @@ class _Row(Parented):
         super(_Row, self).__init__(parent)
         self._tr = tr
 
-    @lazyproperty
+    @property
     def cells(self):
         """
         Sequence of |_Cell| instances corresponding to cells in this row.
-        Supports ``len()``, iteration and indexed access.
         """
-        return _RowCells(self._tr, self)
+        return tuple(self.table.row_cells(self._index))
 
-
-class _RowCells(Parented):
-    """
-    Sequence of |_Cell| instances corresponding to the cells in a table row.
-    """
-    def __init__(self, tr, parent):
-        super(_RowCells, self).__init__(parent)
-        self._tr = tr
-
-    def __getitem__(self, idx):
+    @property
+    def table(self):
         """
-        Provide indexed access, (e.g. 'cells[0]')
+        Reference to the |Table| object this row belongs to.
         """
-        try:
-            tc = self._tr.tc_lst[idx]
-        except IndexError:
-            msg = "cell index [%d] is out of range" % idx
-            raise IndexError(msg)
-        return _Cell(tc, self)
+        return self._parent.table
 
-    def __iter__(self):
-        return (_Cell(tc, self) for tc in self._tr.tc_lst)
-
-    def __len__(self):
-        return len(self._tr.tc_lst)
+    @property
+    def _index(self):
+        """
+        Index of this row in its table, starting from zero.
+        """
+        return self._tr.tr_idx
 
 
 class _Rows(Parented):
@@ -340,3 +379,10 @@ class _Rows(Parented):
 
     def __len__(self):
         return len(self._tbl.tr_lst)
+
+    @property
+    def table(self):
+        """
+        Reference to the |Table| object this row collection belongs to.
+        """
+        return self._parent.table
