@@ -8,15 +8,17 @@ from __future__ import absolute_import, print_function, unicode_literals
 
 import pytest
 
+from docx.oxml import parse_xml
+from docx.oxml.table import CT_Tc
 from docx.shared import Inches
-from docx.table import (
-    _Cell, _Column, _ColumnCells, _Columns, _Row, _RowCells, _Rows, Table
-)
+from docx.table import _Cell, _Column, _Columns, _Row, _Rows, Table
 from docx.text import Paragraph
 
 from .oxml.unitdata.table import a_gridCol, a_tbl, a_tblGrid, a_tc, a_tr
 from .oxml.unitdata.text import a_p
 from .unitutil.cxml import element, xml
+from .unitutil.file import snippet_seq
+from .unitutil.mock import instance_mock, property_mock
 
 
 class DescribeTable(object):
@@ -38,6 +40,16 @@ class DescribeTable(object):
                 tc = tr.tc_lst[col_idx]
                 assert tc is cell._tc
 
+    def it_provides_access_to_the_cells_in_a_column(self, col_cells_fixture):
+        table, column_idx, expected_cells = col_cells_fixture
+        column_cells = table.column_cells(column_idx)
+        assert column_cells == expected_cells
+
+    def it_provides_access_to_the_cells_in_a_row(self, row_cells_fixture):
+        table, row_idx, expected_cells = row_cells_fixture
+        row_cells = table.row_cells(row_idx)
+        assert row_cells == expected_cells
+
     def it_can_add_a_row(self, add_row_fixture):
         table, expected_xml = add_row_fixture
         row = table.add_row()
@@ -70,6 +82,25 @@ class DescribeTable(object):
         table.autofit = new_value
         assert table._tbl.xml == expected_xml
 
+    def it_knows_it_is_the_table_its_children_belong_to(self, table_fixture):
+        table = table_fixture
+        assert table.table is table
+
+    def it_knows_its_column_count_to_help(self, column_count_fixture):
+        table, expected_value = column_count_fixture
+        column_count = table._column_count
+        assert column_count == expected_value
+
+    def it_provides_access_to_its_cells_to_help(self, cells_fixture):
+        table, cell_count, unique_count, matches = cells_fixture
+        cells = table._cells
+        assert len(cells) == cell_count
+        assert len(set(cells)) == unique_count
+        for matching_idxs in matches:
+            comparator_idx = matching_idxs[0]
+            for idx in matching_idxs[1:]:
+                assert cells[idx] is cells[comparator_idx]
+
     # fixtures -------------------------------------------------------
 
     @pytest.fixture
@@ -115,6 +146,49 @@ class DescribeTable(object):
         expected_xml = xml(expected_tbl_cxml)
         return table, new_value, expected_xml
 
+    @pytest.fixture(params=[
+        (0, 9, 9, ()),
+        (1, 9, 8, ((0, 1),)),
+        (2, 9, 8, ((1, 4),)),
+        (3, 9, 6, ((0, 1, 3, 4),)),
+        (4, 9, 4, ((0, 1), (3, 6), (4, 5, 7, 8))),
+    ])
+    def cells_fixture(self, request):
+        snippet_idx, cell_count, unique_count, matches = request.param
+        tbl_xml = snippet_seq('tbl-cells')[snippet_idx]
+        table = Table(parse_xml(tbl_xml), None)
+        return table, cell_count, unique_count, matches
+
+    @pytest.fixture
+    def col_cells_fixture(self, _cells_, _column_count_):
+        table = Table(None, None)
+        _cells_.return_value = [0, 1, 2, 3, 4, 5, 6, 7, 8]
+        _column_count_.return_value = 3
+        column_idx = 1
+        expected_cells = [1, 4, 7]
+        return table, column_idx, expected_cells
+
+    @pytest.fixture
+    def column_count_fixture(self):
+        tbl_cxml = 'w:tbl/w:tblGrid/(w:gridCol,w:gridCol,w:gridCol)'
+        expected_value = 3
+        table = Table(element(tbl_cxml), None)
+        return table, expected_value
+
+    @pytest.fixture
+    def row_cells_fixture(self, _cells_, _column_count_):
+        table = Table(None, None)
+        _cells_.return_value = [0, 1, 2, 3, 4, 5, 6, 7, 8]
+        _column_count_.return_value = 3
+        row_idx = 1
+        expected_cells = [3, 4, 5]
+        return table, row_idx, expected_cells
+
+    @pytest.fixture
+    def table_fixture(self):
+        table = Table(None, None)
+        return table
+
     @pytest.fixture(params=[
         ('w:tbl/w:tblPr', None),
         ('w:tbl/w:tblPr/w:tblStyle{w:val=foobar}', 'foobar'),
@@ -142,6 +216,14 @@ class DescribeTable(object):
 
     # fixture components ---------------------------------------------
 
+    @pytest.fixture
+    def _cells_(self, request):
+        return property_mock(request, Table, '_cells')
+
+    @pytest.fixture
+    def _column_count_(self, request):
+        return property_mock(request, Table, '_column_count')
+
     @pytest.fixture
     def table(self):
         tbl = _tbl_bldr(rows=2, cols=2).element
@@ -151,17 +233,26 @@ class DescribeTable(object):
 
 class Describe_Cell(object):
 
-    def it_can_add_a_paragraph(self, add_paragraph_fixture):
-        cell, expected_xml = add_paragraph_fixture
-        p = cell.add_paragraph()
-        assert cell._tc.xml == expected_xml
-        assert isinstance(p, Paragraph)
+    def it_knows_what_text_it_contains(self, text_get_fixture):
+        cell, expected_text = text_get_fixture
+        text = cell.text
+        assert text == expected_text
 
-    def it_can_add_a_table(self, add_table_fixture):
-        cell, expected_xml = add_table_fixture
-        table = cell.add_table(rows=0, cols=0)
+    def it_can_replace_its_content_with_a_string_of_text(
+            self, text_set_fixture):
+        cell, text, expected_xml = text_set_fixture
+        cell.text = text
+        assert cell._tc.xml == expected_xml
+
+    def it_knows_its_width_in_EMU(self, width_get_fixture):
+        cell, expected_width = width_get_fixture
+        assert cell.width == expected_width
+
+    def it_can_change_its_width(self, width_set_fixture):
+        cell, value, expected_xml = width_set_fixture
+        cell.width = value
+        assert cell.width == value
         assert cell._tc.xml == expected_xml
-        assert isinstance(table, Table)
 
     def it_provides_access_to_the_paragraphs_it_contains(
             self, paragraphs_fixture):
@@ -187,21 +278,25 @@ class Describe_Cell(object):
             count += 1
         assert count == expected_count
 
-    def it_can_replace_its_content_with_a_string_of_text(
-            self, text_set_fixture):
-        cell, text, expected_xml = text_set_fixture
-        cell.text = text
+    def it_can_add_a_paragraph(self, add_paragraph_fixture):
+        cell, expected_xml = add_paragraph_fixture
+        p = cell.add_paragraph()
         assert cell._tc.xml == expected_xml
+        assert isinstance(p, Paragraph)
 
-    def it_knows_its_width_in_EMU(self, width_get_fixture):
-        cell, expected_width = width_get_fixture
-        assert cell.width == expected_width
-
-    def it_can_change_its_width(self, width_set_fixture):
-        cell, value, expected_xml = width_set_fixture
-        cell.width = value
-        assert cell.width == value
+    def it_can_add_a_table(self, add_table_fixture):
+        cell, expected_xml = add_table_fixture
+        table = cell.add_table(rows=0, cols=0)
         assert cell._tc.xml == expected_xml
+        assert isinstance(table, Table)
+
+    def it_can_merge_itself_with_other_cells(self, merge_fixture):
+        cell, other_cell, merged_tc_ = merge_fixture
+        merged_cell = cell.merge(other_cell)
+        cell._tc.merge.assert_called_once_with(other_cell._tc)
+        assert isinstance(merged_cell, _Cell)
+        assert merged_cell._tc is merged_tc_
+        assert merged_cell._parent is cell._parent
 
     # fixtures -------------------------------------------------------
 
@@ -231,6 +326,12 @@ class Describe_Cell(object):
         expected_xml = xml(after_tc_cxml)
         return cell, expected_xml
 
+    @pytest.fixture
+    def merge_fixture(self, tc_, tc_2_, parent_, merged_tc_):
+        cell, other_cell = _Cell(tc_, parent_), _Cell(tc_2_, parent_)
+        tc_.merge.return_value = merged_tc_
+        return cell, other_cell, merged_tc_
+
     @pytest.fixture
     def paragraphs_fixture(self):
         return _Cell(element('w:tc/(w:p, w:p)'), None)
@@ -247,6 +348,19 @@ class Describe_Cell(object):
         cell = _Cell(element(cell_cxml), None)
         return cell, expected_count
 
+    @pytest.fixture(params=[
+        ('w:tc',                                     ''),
+        ('w:tc/w:p/w:r/w:t"foobar"',                 'foobar'),
+        ('w:tc/(w:p/w:r/w:t"foo",w:p/w:r/w:t"bar")', 'foo\nbar'),
+        ('w:tc/(w:tcPr,w:p/w:r/w:t"foobar")',        'foobar'),
+        ('w:tc/w:p/w:r/(w:t"fo",w:tab,w:t"ob",w:br,w:t"ar",w:br)',
+         'fo\tob\nar\n'),
+    ])
+    def text_get_fixture(self, request):
+        tc_cxml, expected_text = request.param
+        cell = _Cell(element(tc_cxml), None)
+        return cell, expected_text
+
     @pytest.fixture(params=[
         ('w:tc/w:p', 'foobar',
          'w:tc/w:p/w:r/w:t"foobar"'),
@@ -284,13 +398,36 @@ class Describe_Cell(object):
         expected_xml = xml(expected_cxml)
         return cell, new_value, expected_xml
 
+    # fixture components ---------------------------------------------
+
+    @pytest.fixture
+    def merged_tc_(self, request):
+        return instance_mock(request, CT_Tc)
+
+    @pytest.fixture
+    def parent_(self, request):
+        return instance_mock(request, Table)
+
+    @pytest.fixture
+    def tc_(self, request):
+        return instance_mock(request, CT_Tc)
+
+    @pytest.fixture
+    def tc_2_(self, request):
+        return instance_mock(request, CT_Tc)
+
 
 class Describe_Column(object):
 
-    def it_provides_access_to_the_column_cells(self):
-        column = _Column(None, None, None)
+    def it_provides_access_to_its_cells(self, cells_fixture):
+        column, column_idx, expected_cells = cells_fixture
         cells = column.cells
-        assert isinstance(cells, _ColumnCells)
+        column.table.column_cells.assert_called_once_with(column_idx)
+        assert cells == expected_cells
+
+    def it_provides_access_to_the_table_it_belongs_to(self, table_fixture):
+        column, table_ = table_fixture
+        assert column.table is table_
 
     def it_knows_its_width_in_EMU(self, width_get_fixture):
         column, expected_width = width_get_fixture
@@ -302,8 +439,33 @@ class Describe_Column(object):
         assert column.width == value
         assert column._gridCol.xml == expected_xml
 
+    def it_knows_its_index_in_table_to_help(self, index_fixture):
+        column, expected_idx = index_fixture
+        assert column._index == expected_idx
+
     # fixtures -------------------------------------------------------
 
+    @pytest.fixture
+    def cells_fixture(self, _index_, table_prop_, table_):
+        column = _Column(None, None)
+        _index_.return_value = column_idx = 4
+        expected_cells = (3, 2, 1)
+        table_.column_cells.return_value = list(expected_cells)
+        return column, column_idx, expected_cells
+
+    @pytest.fixture
+    def index_fixture(self):
+        tbl = element('w:tbl/w:tblGrid/(w:gridCol,w:gridCol,w:gridCol)')
+        gridCol, expected_idx = tbl.tblGrid[1], 1
+        column = _Column(gridCol, None)
+        return column, expected_idx
+
+    @pytest.fixture
+    def table_fixture(self, parent_, table_):
+        column = _Column(None, parent_)
+        parent_.table = table_
+        return column, table_
+
     @pytest.fixture(params=[
         ('w:gridCol{w:w=4242}',   2693670),
         ('w:gridCol{w:w=1440}',    914400),
@@ -314,7 +476,7 @@ class Describe_Column(object):
     ])
     def width_get_fixture(self, request):
         gridCol_cxml, expected_width = request.param
-        column = _Column(element(gridCol_cxml), None, None)
+        column = _Column(element(gridCol_cxml), None)
         return column, expected_width
 
     @pytest.fixture(params=[
@@ -325,49 +487,27 @@ class Describe_Column(object):
     ])
     def width_set_fixture(self, request):
         gridCol_cxml, new_value, expected_cxml = request.param
-        column = _Column(element(gridCol_cxml), None, None)
+        column = _Column(element(gridCol_cxml), None)
         expected_xml = xml(expected_cxml)
         return column, new_value, expected_xml
 
-
-class Describe_ColumnCells(object):
-
-    def it_knows_how_many_cells_it_contains(self, cells_fixture):
-        cells, cell_count = cells_fixture
-        assert len(cells) == cell_count
-
-    def it_can_iterate_over_its__Cell_instances(self, cells_fixture):
-        cells, cell_count = cells_fixture
-        actual_count = 0
-        for cell in cells:
-            assert isinstance(cell, _Cell)
-            actual_count += 1
-        assert actual_count == cell_count
-
-    def it_provides_indexed_access_to_cells(self, cells_fixture):
-        cells, cell_count = cells_fixture
-        for idx in range(-cell_count, cell_count):
-            cell = cells[idx]
-            assert isinstance(cell, _Cell)
-
-    def it_raises_on_indexed_access_out_of_range(self, cells_fixture):
-        cells, cell_count = cells_fixture
-        too_low = -1 - cell_count
-        too_high = cell_count
-        with pytest.raises(IndexError):
-            cells[too_low]
-        with pytest.raises(IndexError):
-            cells[too_high]
-
-    # fixtures -------------------------------------------------------
+    # fixture components ---------------------------------------------
 
     @pytest.fixture
-    def cells_fixture(self):
-        cell_count = 2
-        tbl = _tbl_bldr(rows=cell_count, cols=1).element
-        gridCol = tbl.tblGrid.gridCol_lst[0]
-        cells = _ColumnCells(tbl, gridCol, None)
-        return cells, cell_count
+    def _index_(self, request):
+        return property_mock(request, _Column, '_index')
+
+    @pytest.fixture
+    def parent_(self, request):
+        return instance_mock(request, Table)
+
+    @pytest.fixture
+    def table_(self, request):
+        return instance_mock(request, Table)
+
+    @pytest.fixture
+    def table_prop_(self, request, table_):
+        return property_mock(request, _Column, 'table', return_value=table_)
 
 
 class Describe_Columns(object):
@@ -399,6 +539,10 @@ class Describe_Columns(object):
         with pytest.raises(IndexError):
             columns[too_high]
 
+    def it_provides_access_to_the_table_it_belongs_to(self, table_fixture):
+        columns, table_ = table_fixture
+        assert columns.table is table_
+
     # fixtures -------------------------------------------------------
 
     @pytest.fixture
@@ -408,51 +552,75 @@ class Describe_Columns(object):
         columns = _Columns(tbl, None)
         return columns, column_count
 
+    @pytest.fixture
+    def table_fixture(self, table_):
+        columns = _Columns(None, table_)
+        table_.table = table_
+        return columns, table_
+
+    # fixture components ---------------------------------------------
+
+    @pytest.fixture
+    def table_(self, request):
+        return instance_mock(request, Table)
+
 
 class Describe_Row(object):
 
-    def it_provides_access_to_the_row_cells(self):
-        row = _Row(element('w:tr'), None)
+    def it_provides_access_to_its_cells(self, cells_fixture):
+        row, row_idx, expected_cells = cells_fixture
         cells = row.cells
-        assert isinstance(cells, _RowCells)
+        row.table.row_cells.assert_called_once_with(row_idx)
+        assert cells == expected_cells
 
+    def it_provides_access_to_the_table_it_belongs_to(self, table_fixture):
+        row, table_ = table_fixture
+        assert row.table is table_
 
-class Describe_RowCells(object):
-
-    def it_knows_how_many_cells_it_contains(self, cell_count_fixture):
-        cells, cell_count = cell_count_fixture
-        assert len(cells) == cell_count
-
-    def it_can_iterate_over_its__Cell_instances(self, cell_count_fixture):
-        cells, cell_count = cell_count_fixture
-        actual_count = 0
-        for cell in cells:
-            assert isinstance(cell, _Cell)
-            actual_count += 1
-        assert actual_count == cell_count
-
-    def it_provides_indexed_access_to_cells(self, cell_count_fixture):
-        cells, cell_count = cell_count_fixture
-        for idx in range(-cell_count, cell_count):
-            cell = cells[idx]
-            assert isinstance(cell, _Cell)
-
-    def it_raises_on_indexed_access_out_of_range(self, cell_count_fixture):
-        cells, cell_count = cell_count_fixture
-        too_low = -1 - cell_count
-        too_high = cell_count
-        with pytest.raises(IndexError):
-            cells[too_low]
-        with pytest.raises(IndexError):
-            cells[too_high]
+    def it_knows_its_index_in_table_to_help(self, idx_fixture):
+        row, expected_idx = idx_fixture
+        assert row._index == expected_idx
 
     # fixtures -------------------------------------------------------
 
     @pytest.fixture
-    def cell_count_fixture(self):
-        cells = _RowCells(element('w:tr/(w:tc, w:tc)'), None)
-        cell_count = 2
-        return cells, cell_count
+    def cells_fixture(self, _index_, table_prop_, table_):
+        row = _Row(None, None)
+        _index_.return_value = row_idx = 6
+        expected_cells = (1, 2, 3)
+        table_.row_cells.return_value = list(expected_cells)
+        return row, row_idx, expected_cells
+
+    @pytest.fixture
+    def idx_fixture(self):
+        tbl = element('w:tbl/(w:tr,w:tr,w:tr)')
+        tr, expected_idx = tbl[1], 1
+        row = _Row(tr, None)
+        return row, expected_idx
+
+    @pytest.fixture
+    def table_fixture(self, parent_, table_):
+        row = _Row(None, parent_)
+        parent_.table = table_
+        return row, table_
+
+    # fixture components ---------------------------------------------
+
+    @pytest.fixture
+    def _index_(self, request):
+        return property_mock(request, _Row, '_index')
+
+    @pytest.fixture
+    def parent_(self, request):
+        return instance_mock(request, Table)
+
+    @pytest.fixture
+    def table_(self, request):
+        return instance_mock(request, Table)
+
+    @pytest.fixture
+    def table_prop_(self, request, table_):
+        return property_mock(request, _Row, 'table', return_value=table_)
 
 
 class Describe_Rows(object):
@@ -484,6 +652,10 @@ class Describe_Rows(object):
             too_high = row_count
             rows[too_high]
 
+    def it_provides_access_to_the_table_it_belongs_to(self, table_fixture):
+        rows, table_ = table_fixture
+        assert rows.table is table_
+
     # fixtures -------------------------------------------------------
 
     @pytest.fixture
@@ -493,6 +665,18 @@ class Describe_Rows(object):
         rows = _Rows(tbl, None)
         return rows, row_count
 
+    @pytest.fixture
+    def table_fixture(self, table_):
+        rows = _Rows(None, table_)
+        table_.table = table_
+        return rows, table_
+
+    # fixture components ---------------------------------------------
+
+    @pytest.fixture
+    def table_(self, request):
+        return instance_mock(request, Table)
+
 
 # fixtures -----------------------------------------------------------
 
